'use strict';

var glob = require('../util/glob');
var path = require('path');

var rRequire = /^(?:var\s+?([\w$]+)\s*?=\s*?)?require\s*?\(\s*?(['"])(.+)\2\s*?\)\s*?;?\s*?$[\n\r]*/mg;
var rModuleWrapper = /^(define|require)\s*?\(/m;

var wrapModular = function wrapModular(repoLocation, repoConfig, filePath, fileContent) {
    var moduleExports = [],
        moduleIds = [],
        moduleId = undefined;
    if (!repoConfig.modular || repoConfig.modular.ignore && glob.match(filePath, repoConfig.modular.ignore)) {
        return fileContent;
    }

    fileContent = fileContent.replace(rRequire, function (line, me, qoute, mi) {
        moduleExports.push(me || '__' + path.basename(mi) + '__');
        moduleIds.push(mi);
        return '';
    });

    if (!rModuleWrapper.test(fileContent)) {
        moduleIds.push('require', 'exports', 'module');
        moduleExports.push('require', 'exports', 'module');
        moduleId = filePath.slice(0, -path.extname(filePath).length).replace(/^[^\/]+\//, '');
        if (repoConfig.modular.idprefix) {
            moduleId = repoConfig.modular.idprefix + '/' + moduleId;
        }
        fileContent = "define('" + moduleId + "', ['" + moduleIds.join("','") + "'], function (" + moduleExports.join(",") + ") {\n" + fileContent + "});";
    }
    return fileContent;
};

module.exports = function (file, callback) {
    var filedata = wrapModular(file.get('workdir'), file.get('config'), file.get('filename'), file.get('filedata'));
    file.set('filedata', filedata);
    callback();
};
