/* eslint no-eq-null:0 */
'use strict';

var fs = require('fs-extra');
var path = require('path');
var pkg = require('../../package.json');

var homedir = process.platform === 'win32' ? path.join(process.env.LOCALAPPDATA, pkg.name) : path.join(process.env.HOME, '.' + pkg.name);

exports.get = function (type, key) {
    var filename = path.join(homedir, type) + '.json',
        filedata = undefined;
    if (fs.existsSync(filename)) {
        filedata = fs.readJSONSync(filename, { throws: false }) || {};
    } else {
        filedata = {};
    }
    return key ? filedata[key] : filedata;
};

exports.set = function (type, key, value) {
    if (!key) {
        return;
    }
    var filename = path.join(homedir, type) + '.json';
    var filedata = exports.get(type);
    if (typeof key === 'object') {
        filedata = key;
    } else if (value == null) {
        delete filedata[key];
    } else {
        filedata[key] = value;
    }
    fs.outputJSONSync(filename, filedata, { spaces: 2 });
};

exports.file = function (name, content) {
    var filename = path.join(homedir, 'tmp', name);
    if (content === undefined) {
        return fs.existsSync(filename) ? fs.readFileSync(filename, { encoding: 'utf8' }) : '';
    } else if (content == null) {
        fs.unlinkSync(filename);
    } else {
        fs.outputFileSync(filename, content);
        return filename;
    }
};
